package explorviz.hpc_monitoring.filter.reconstruction;

import java.util.ArrayList;
import java.util.List;

import explorviz.hpc_monitoring.record.HostApplicationMetaData;
import explorviz.hpc_monitoring.record.event.AbstractOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.AfterFailedOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.AfterOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.BeforeOperationEvent;
import explorviz.hpc_monitoring.record.trace.Trace;

public class TraceBuffer {
	// private static final Comparator<AbstractOperationEvent> COMPARATOR = new
	// AbstractOperationEventComperator();

	private static final int INITIAL_EVENT_CAPACITY = 100;

	private HostApplicationMetaData traceMetadata;
	private final List<AbstractOperationEvent> events = new ArrayList<AbstractOperationEvent>(
			INITIAL_EVENT_CAPACITY);

	private boolean closeable;
	private boolean damaged;

	private int openEvents;

	private long maxLoggingTimestamp = -1;
	private int maxOrderIndex = -1;

	public final void insertEvent(final AbstractOperationEvent event) {
		setMaxLoggingTimestamp(event);
		final int orderIndex = setMaxOrderIndex(event);

		if (event instanceof BeforeOperationEvent) {
			if (orderIndex == 0) {
				closeable = true;
			}
			openEvents++;
		} else if (event instanceof AfterOperationEvent) {
			openEvents--;
		} else if (event instanceof AfterFailedOperationEvent) {
			openEvents--;
		}

		if (!events.add(event)) { // TODO
			System.out.println("Duplicate entry for orderIndex " + orderIndex + " with traceId "
					+ event.getTraceId());
			damaged = true;
		}
	}

	public final long getMaxLoggingTimestamp() {
		return maxLoggingTimestamp;
	}

	private final void setMaxLoggingTimestamp(final AbstractOperationEvent event) {
		final long loggingTimestamp = event.getLoggingTimestamp();
		if (loggingTimestamp > maxLoggingTimestamp) {
			maxLoggingTimestamp = loggingTimestamp;
		}
	}

	private final int setMaxOrderIndex(final AbstractOperationEvent event) {
		final int orderIndex = event.getOrderIndex();
		if (orderIndex > maxOrderIndex) {
			maxOrderIndex = orderIndex;
		}
		return orderIndex;
	}

	public void setTrace(final HostApplicationMetaData trace) {
		if (traceMetadata != null) {
			damaged = true;
			return;
		}
		traceMetadata = trace;
	}

	public final boolean isFinished() {
		return !isInvalid();
	}

	public final boolean isInvalid() {
		return ((openEvents != 0) || events.isEmpty() || ((maxOrderIndex + 1) != events.size())
				|| damaged || !closeable);
	}

	public final Trace toTrace() {
		return new Trace(traceMetadata, events);
	}

	// private static final class AbstractOperationEventComperator implements
	// Comparator<AbstractOperationEvent> {
	// @Override
	// public int compare(final AbstractOperationEvent o1, final
	// AbstractOperationEvent o2) {
	// return o1.getOrderIndex() - o2.getOrderIndex();
	// }
	// }
}
